#def lookup_online():
import yaml
import os
import re
import urllib.request

class Parser():
    def __init__(self, offline=True):
        if offline:
            stream = open(f'{os.path.dirname(__file__)}/tocalls.yaml', 'r')
        else:
            stream = urllib.request.urlopen('https://raw.githubusercontent.com/aprsorg/aprs-deviceid/main/tocalls.yaml')
        self.data = yaml.safe_load(stream)
        stream.close()
    def return_data(self, data): # so we can format data in a standard way
        del data['specific_count']
        return data   
    def lookup(self,call):
        # always check in upper
        call = call.upper()
        #check wildcard matches starting from most specific
        matches = []
        for matching_call in self.data['tocalls']: # loop through all calls in tocalls.txt
            matcher = matching_call['tocall']
            matcher = matcher.replace("?",'\w')
            matcher = matcher.replace("n",'\d')
            matcher = matcher.replace("*",'\w')
            if re.match(matcher, call):
                matched = matching_call
                specific_count = matching_call['tocall'].count("n") + matching_call['tocall'].count("?") + matching_call['tocall'].count("*")
                matched["specific_count"] = specific_count
                matches.append(matched)
        # try to get the most specific
        matches = sorted(matches, key=lambda d: d['specific_count']) 
        if matches:
            return self.return_data(matches[0])
        raise IndexError("Not found in tocalls.yaml")


from . import Parser


import unittest

class TestLookupsMethods(unittest.TestCase):
    def setUp(self):
        self.aprs_tocall = Parser()
    def test_specific(self):
        self.assertEqual(self.aprs_tocall.lookup("APWEEA"), 
                            {
                                'tocall': 'APWEE?',
                                'vendor': 'Tom Keffer and Matthew Wall',
                                'model': 'WeeWX Weather Software',
                                'class': 'software',
                                'os': 'Linux/Unix'
                            }
                        )
    def test_exact(self):
        self.assertEqual(self.aprs_tocall.lookup("APRARX"), 
                            {
                                'tocall': 'APRARX',
                                'vendor': 'Open Source',
                                'model': 'radiosonde_auto_rx',
                                'class': 'software',
                                'os': 'Linux/Unix'
                            }
                        )
    def test_lower(self):
        self.assertEqual(self.aprs_tocall.lookup("aprarx"), 
                            {
                                'tocall': 'APRARX',
                                'vendor': 'Open Source',
                                'model': 'radiosonde_auto_rx',
                                'class': 'software',
                                'os': 'Linux/Unix'
                            }
                        )
    def test_wild(self):
        self.assertEqual(self.aprs_tocall.lookup("APTTAA"), 
                            {
                                'tocall': 'APTT*',
                                'vendor': 'Byonics',
                                'model': 'TinyTrak',
                                'class': 'tracker'
                            }
                        )        
    def test_n(self):
        self.assertEqual(self.aprs_tocall.lookup("APCCCD"), 
                            {
                                'tocall': 'APC???',
                                'vendor': 'Rob Wittner, KZ5RW',
                                'model': 'APRS/CE',
                                'class': 'app'
                            }
                        )   
    def test_n2(self):
        self.assertEqual(self.aprs_tocall.lookup("AP123U"), 
                            {
                                'tocall': 'APnnnU',
                                'vendor': 'Painter Engineering',
                                'model': 'uSmartDigi Digipeater',
                                'class': 'digi'
                            }
                        )     
    def test_n3(self):
        self.assertEqual(self.aprs_tocall.lookup("AP123D"), 
                            {
                                'tocall': 'APnnnD',
                                'vendor': 'Painter Engineering',
                                'model': 'uSmartDigi D-Gate',
                                'class': 'dstar'
                            }
                        )    
class TestLookupsOnlineMethods(unittest.TestCase):
    def setUp(self):
        self.aprs_tocall = Parser(offline=False)
    def test_specific(self):
        self.assertEqual(self.aprs_tocall.lookup("APWEEA"), 
                            {
                                'tocall': 'APWEE?',
                                'vendor': 'Tom Keffer and Matthew Wall',
                                'model': 'WeeWX Weather Software',
                                'class': 'software',
                                'os': 'Linux/Unix'
                            }
                        )
    def test_exact(self):
        self.assertEqual(self.aprs_tocall.lookup("APRARX"), 
                            {
                                'tocall': 'APRARX',
                                'vendor': 'Open Source',
                                'model': 'radiosonde_auto_rx',
                                'class': 'software',
                                'os': 'Linux/Unix'
                            }
                        )
    def test_lower(self):
        self.assertEqual(self.aprs_tocall.lookup("aprarx"), 
                            {
                                'tocall': 'APRARX',
                                'vendor': 'Open Source',
                                'model': 'radiosonde_auto_rx',
                                'class': 'software',
                                'os': 'Linux/Unix'
                            }
                        )
if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aprs_tocall']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0.1,<7.0.0']

entry_points = \
{'console_scripts': ['aprs_tocall = aprs_tocall:__main__.main']}

setup_kwargs = {
    'name': 'aprs-tocall',
    'version': '0.1.2',
    'description': 'Parses APRS "tocalls" or tocalls.txt / tocalls.yaml easily in Python',
    'long_description': 'aprs_tocall\n==\n\nInstall\n--\n\n```sh\npip install aprs_tocall\n```\n\nUsage\n--\n```python\nfrom aprs_tocall import Parser\ntocalls = Parser(offline=False) \n# Setting offline to False will use the online \n# yaml file, while offline=True (default) will \n# use the distributed version\ntocalls.lookup("APWEEA")\n# returns \n# {\n#   \'tocall\': \'APWEE?\',\n#   \'vendor\': \'Tom Keffer and Matthew Wall\',\n#   \'model\': \'WeeWX Weather Software\',\n#   \'class\': \'software\', \n#   \'os\': \'Linux/Unix\'\n# }\n```\n',
    'author': 'xss',
    'author_email': 'michaela@michaela.lgbt',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/xssfox/aprs_tocall',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from . import Parser
import argparse
import sys

def main():
    cli = argparse.ArgumentParser(
                    prog='aprs_tocall',
                    description='Looks up APRS tocalls')
    cli.add_argument('tocall', type=str, help="The APRS tocall to lookup")  
    cli.add_argument('--online', help="Use online rather than distributed version",
                    action='store_true', default=False)
    args = cli.parse_args()
    aprs_tocall = Parser(offline=not args.online)
    
    try:
        data = aprs_tocall.lookup(args.tocall)
        for key in data:
            print(f"{key}={data[key]}")
    except IndexError:
        sys.stderr.write("Could not find call in tocalls.yaml\n")
if __name__ == '__main__':
    main()